import React, { Component, PropTypes } from 'react';
import Bars from './bars';
import YAxis, { YAxisDefault, YAxisPropTypesShape } from '../YAxis';
import XAxis, { XAxisDefault, XAxisPropTypesShape } from '../XAxis';
import { getColorScale } from '../utils/colorScale';

const scale = require('d3-scale');
const _ = require('d3-array');

class BarChart extends Component {

  static propTypes = {
    data: PropTypes.arrayOf(PropTypes.object).isRequired,
    animation: PropTypes.shape({
      duration: PropTypes.number
    }),
    background: PropTypes.string,
    bandsPaddingInner: PropTypes.number,
    bandsPaddingOuter: PropTypes.number,
    baseLine: PropTypes.number,
    chartClassName: PropTypes.string,
    color: PropTypes.oneOfType([
      PropTypes.string,
      PropTypes.array
    ]),
    height: PropTypes.number,
    labels: PropTypes.shape({
      color: PropTypes.string,
      digitsAfterPoint: PropTypes.number,
      disable: PropTypes.bool,
      fontSize: PropTypes.string
    }),
    padding: PropTypes.shape({
      bottom: PropTypes.number,
      left: PropTypes.number,
      right: PropTypes.number,
      top: PropTypes.number
    }),
    paddingInner: PropTypes.shape({
      bottom: PropTypes.number,
      left: PropTypes.number,
      right: PropTypes.number,
      top: PropTypes.number
    }),
    width: PropTypes.number,
    xAxis: XAxisPropTypesShape,
    yAxis: YAxisPropTypesShape
  };

  static defaultProps = {
    background: '#eee',
    bandsPaddingInner: 0.2,
    baseLine: 0,
    chartClassName: 'r3-barchart',
    color: '#009688',
    height: 300,
    width: 400
  };

  constructor(props) {
    super(props);
    this.conf = {
      animation: {
        duration: 500
      },
      labels: {
        color: '#333',
        digitsAfterPoint: 0,
        disable: false,
        fontSize: '12px'
      },
      padding: {
        bottom: 10,
        left: 10,
        right: 15,
        top: 20
      },
      paddingInner: {
        bottom: 20,
        left: 20,
        right: 5,
        top: 10
      },
      xAxis: Object.assign({}, XAxisDefault, {
        className: 'r3-barchart-xaxis',
        height: 30
      }),
      yAxis: Object.assign({}, YAxisDefault, {
        className: 'r3-barchart-yaxis',
        width: 35
      })
    };
  }

  getYDomain() {
    const { data, baseLine } = this.props;
    let min = _.min(data, v => v.value);
    let max = _.max(data, v => v.value);

    min = min < baseLine ? min : baseLine;
    max = max > baseLine ? max : baseLine;
    return [min, max];
  }

  getStyle() {
    const { background } = this.props;
    return {
      background
    };
  }

  render() {
    const {
      bandsPaddingInner,
      baseLine,
      chartClassName,
      color,
      data,
      height,
      width
    } = this.props;

    const animation = Object.assign({}, this.conf.animation, this.props.animation);
    const labels = Object.assign({}, this.conf.labels, this.props.labels);
    const padding = Object.assign({}, this.conf.padding, this.props.padding);
    const paddingInner = Object.assign({}, this.conf.paddingInner, this.props.paddingInner);
    const xAxis = Object.assign({}, this.conf.xAxis, this.props.xAxis);
    const yAxis = Object.assign({}, this.conf.yAxis, this.props.yAxis);

    // if axis hide then it height/width is 0
    xAxis.height = xAxis.disable ? 0 : xAxis.height;
    yAxis.width = yAxis.disable ? 0 : yAxis.width;

    // width & height available area of the chart
    const barsWidth = width - yAxis.width - padding.left - padding.right;
    const barsHeight = height - xAxis.height - padding.top - padding.bottom;

    const colorScale = getColorScale(color);

    const xScale = scale.scaleBand()
      .domain(data.map(v => v.name))
      .rangeRound([paddingInner.left, barsWidth - paddingInner.right])
      .paddingInner(bandsPaddingInner);

    const yScale = scale.scaleLinear()
      .domain(this.getYDomain())
      .range([paddingInner.bottom, barsHeight - paddingInner.top]);

    // get array division, in accordance with a given amount,
    const yTicks = yScale.ticks(yAxis.tickCount);

    return (
      <svg
        className={chartClassName}
        height={height}
        style={this.getStyle()}
        width={width}
      >

        <g transform={`translate(${padding.left},${padding.top})`}>
          <Bars
            animation={animation}
            baseLine={baseLine}
            colorScale={colorScale}
            data={data}
            dx={yAxis.width}
            height={barsHeight}
            labels={labels}
            xScale={xScale}
            yScale={yScale}
          />

          <XAxis
            className={xAxis.className}
            disable={xAxis.disable}
            dx={yAxis.width}
            dy={barsHeight}
            fontColor={xAxis.fontColor}
            fontSize={xAxis.fontSize}
            label={xAxis.label}
            labelColor={xAxis.labelColor}
            labelSize={xAxis.labelSize}
            lineColor={xAxis.lineColor}
            lineWidth={xAxis.lineWidth}
            tickColor={xAxis.tickColor}
            ticks={xScale.domain()}
            tickWidth={xAxis.tickWidth}
            tickShift={xScale.bandwidth() / 2}
            width={barsWidth}
            xScale={xScale}
          />

          <YAxis
            className={yAxis.className}
            disable={yAxis.disable}
            dx={yAxis.width}
            fontColor={yAxis.fontColor}
            fontSize={yAxis.fontSize}
            label={yAxis.label}
            labelColor={yAxis.labelColor}
            labelSize={yAxis.labelSize}
            height={barsHeight}
            lineColor={yAxis.lineColor}
            lineWidth={yAxis.lineWidth}
            ticks={yTicks}
            tickColor={yAxis.tickColor}
            tickWidth={yAxis.tickWidth}
            yScale={yScale}
          />

        </g>

      </svg>
    );
  }

}

export default BarChart;
